@@ -39,6 +39,7 @@ from pypowervm.helpers import log_helper as log_hlp
 from pypowervm.helpers import vios_busy as vio_hlp
 from pypowervm.tasks import cna as pvm_cna
 from pypowervm.tasks import memory as pvm_mem
+from pypowervm.tasks.monitor import host_cpu as pvm_hcpu
 from pypowervm.tasks import partition as pvm_par
 from pypowervm.tasks import power_opts as pvm_popts
 from pypowervm.tasks import scsi_mapper as pvm_smap
@@ -94,7 +95,11 @@ class PowerVMDriver(driver.ComputeDriver):
         "supports_recreate": True,
         "supports_migrate_to_same_host": False,
         "supports_attach_interface": True,
-        "supports_device_tagging": False
+        "supports_device_tagging": False,
+        "supports_tagged_attach_interface": False,
+        "supports_tagged_attach_volume": False,
+        "supports_extend_volume": True,
+        "supports_multiattach": False,
     }
 
     def __init__(self, virtapi):
@@ -130,8 +135,8 @@ class PowerVMDriver(driver.ComputeDriver):
         self._setup_rebuild_store()
 
         # Init Host CPU Statistics
-        self.host_cpu_stats = pvm_host.HostCPUStats(self.adapter,
-                                                    self.host_uuid)
+        self.host_cpu_cache = pvm_hcpu.HostCPUMetricCache(self.adapter,
+                                                          self.host_uuid)
 
         # Cache for instance overhead.
         # Key: max_mem (int MB)
@@ -222,19 +227,12 @@ class PowerVMDriver(driver.ComputeDriver):
                   'name': instance.name}, instance=instance)
 
     def get_info(self, instance):
-        """Get the current status of an instance, by name (not ID!)
+        """Get the current status of an instance.
 
-        Returns a dict containing:
-
-        :state:           the running state, one of the power_state codes
-        :max_mem:         (int) the maximum memory in KBytes allowed
-        :mem:             (int) the memory in KBytes used by the domain
-        :num_cpu:         (int) the number of virtual CPUs for the domain
-        :cpu_time:        (int) the CPU time used in nanoseconds
+        :param instance: nova.objects.instance.Instance object
+        :returns: An InstanceInfo object
         """
-        info = vm.InstanceInfo(self.adapter, instance.name,
-                               vm.get_pvm_uuid(instance))
-        return info
+        return vm.get_vm_info(self.adapter, instance)
 
     def instance_exists(self, instance):
         """Checks existence of an instance on the host.
@@ -298,7 +296,16 @@ class PowerVMDriver(driver.ComputeDriver):
 
     def get_host_cpu_stats(self):
         """Return the current CPU state of the host."""
-        return self.host_cpu_stats.get_host_cpu_stats()
+        self.host_cpu_cache.refresh()
+        return {
+            'kernel': self.host_cpu_cache.total_fw_cycles,
+            'user': self.host_cpu_cache.total_user_cycles,
+            'idle': (self.host_cpu_cache.total_cycles -
+                     self.host_cpu_cache.total_user_cycles -
+                     self.host_cpu_cache.total_fw_cycles),
+            # Not reported by PowerVM
+            'iowait': 0,
+            'frequency': self.host_cpu_cache.cpu_freq}
 
     def instance_on_disk(self, instance):
         """Checks access of instance files on the host.
@@ -340,7 +347,8 @@ class PowerVMDriver(driver.ComputeDriver):
         return False
 
     def spawn(self, context, instance, image_meta, injected_files,
-              admin_password, network_info=None, block_device_info=None):
+              admin_password, allocations, network_info=None,
+              block_device_info=None):
         """Create a new instance/VM/domain on the virtualization platform.
 
         Once this successfully completes, the instance should be
@@ -358,6 +366,9 @@ class PowerVMDriver(driver.ComputeDriver):
             The metadata of the image of the instance.
         :param injected_files: User files to inject into instance.
         :param admin_password: Administrator password to set in instance.
+        :param allocations: Information about resources allocated to the
+                            instance via placement, of the form returned by
+                            SchedulerReportClient.get_allocations_for_consumer.
         :param network_info: instance network information
         :param block_device_info: Information about block devices to be
                                   attached to the instance.
@@ -381,7 +392,7 @@ class PowerVMDriver(driver.ComputeDriver):
         # Build the PowerVM Slot lookup map.  Only the recreate action needs
         # the volume driver iterator (to look up volumes and their client
         # mappings).
-        vol_drv_iter = (self._vol_drv_iter(context, instance, bdms=bdms,
+        vol_drv_iter = (self._vol_drv_iter(context, instance, bdms,
                                            stg_ftsk=stg_ftsk)
                         if recreate else None)
         slot_mgr = slot.build_slot_mgr(
@@ -432,7 +443,7 @@ class PowerVMDriver(driver.ComputeDriver):
         # after all the other I/O.
         if configdrive.required_by(instance) and not recreate:
             flow_spawn.add(tf_stg.CreateAndConnectCfgDrive(
-                self.adapter, self.host_uuid, instance, injected_files,
+                self.adapter, instance, injected_files,
                 network_info, admin_password, stg_ftsk=stg_ftsk))
 
         # Add the transaction manager flow to the end of the 'I/O
@@ -482,7 +493,7 @@ class PowerVMDriver(driver.ComputeDriver):
         :param slot_mgr: A NovaSlotManager.  Used to store/retrieve the client
                          slots used when a volume is attached to a VM.
         """
-        for bdm, vol_drv in self._vol_drv_iter(context, instance, bdms=bdms,
+        for bdm, vol_drv in self._vol_drv_iter(context, instance, bdms,
                                                stg_ftsk=stg_ftsk):
             # First connect the volume.  This will update the
             # connection_info.
@@ -508,7 +519,7 @@ class PowerVMDriver(driver.ComputeDriver):
                          slots used when a volume is detached from a VM.
         """
         # TODO(thorst) Do we need to do something on the disconnect for slots?
-        for bdm, vol_drv in self._vol_drv_iter(context, instance, bdms=bdms,
+        for bdm, vol_drv in self._vol_drv_iter(context, instance, bdms,
                                                stg_ftsk=stg_ftsk):
             flow.add(tf_stg.DisconnectVolume(vol_drv, slot_mgr))
 
@@ -591,8 +602,7 @@ class PowerVMDriver(driver.ComputeDriver):
             # manager.
             if configdrive.required_by(instance):
                 flow.add(tf_stg.DeleteVOpt(
-                    self.adapter, self.host_uuid, instance, pvm_inst_uuid,
-                    stg_ftsk=stg_ftsk))
+                    self.adapter, instance, stg_ftsk=stg_ftsk))
 
             # Determine if there are volumes to disconnect.  If so, remove each
             # volume (within the transaction manager)
@@ -744,6 +754,12 @@ class PowerVMDriver(driver.ComputeDriver):
         # to revise in the future as volume connectors evolve.
         instance.save()
 
+    def extend_volume(self, connection_info, instance):
+        """Resize an attached volume"""
+        vol_drv = vol_attach.build_volume_driver(
+            self.adapter, self.host_uuid, instance, connection_info)
+        vol_drv.extend_volume()
+
     def detach_volume(self, connection_info, instance, mountpoint,
                       encryption=None):
         """Detach the volume attached to the instance."""
@@ -800,6 +816,11 @@ class PowerVMDriver(driver.ComputeDriver):
                                       update.  The driver will raise if this
                                       doesn't match.
         """
+        if not self.disk_dvr.capabilities.get('snapshot'):
+            raise exception.NotSupportedWithOption(
+                message=_("The snapshot operation is not supported in "
+                          "conjunction with a CONF.powervm.disk_driver "
+                          "setting of %s.") % CONF.powervm.disk_driver)
         self._log_operation('snapshot', instance)
 
         # Define the flow
@@ -968,11 +989,11 @@ class PowerVMDriver(driver.ComputeDriver):
         out, err = n_utils.execute('env', 'LANG=C', 'uptime')
         return out
 
-    def attach_interface(self, instance, image_meta, vif):
+    def attach_interface(self, context, instance, image_meta, vif):
         """Attach an interface to the instance."""
         self.plug_vifs(instance, [vif])
 
-    def detach_interface(self, instance, vif):
+    def detach_interface(self, context, instance, vif):
         """Detach an interface from the instance."""
         self.unplug_vifs(instance, [vif])
 
@@ -1028,9 +1049,8 @@ class PowerVMDriver(driver.ComputeDriver):
         try:
             tf_base.run(flow, instance=instance)
         except exception.InstanceNotFound:
-            LOG.exception('VM was not found during unplug operation '
-                          'as it is already possibly deleted.',
-                          instance=instance)
+            LOG.warning('VM was not found during unplug operation as it is '
+                        'already possibly deleted.', instance=instance)
         except Exception:
             LOG.exception("PowerVM error trying to unplug vifs.",
                           instance=instance)
@@ -1081,6 +1101,7 @@ class PowerVMDriver(driver.ComputeDriver):
             self.adapter, self.host_uuid, instance)
         if wwpn_list is not None:
             connector["wwpns"] = wwpn_list
+        connector["multipath"] = CONF.powervm.volume_use_multipath
         connector['host'] = vol_attach.get_hostname_for_volume(instance)
         connector['initiator'] = vol_attach.get_iscsi_initiator(self.adapter)
         return connector
@@ -1144,7 +1165,7 @@ class PowerVMDriver(driver.ComputeDriver):
             slot_mgr = slot.build_slot_mgr(
                 instance, self.store_api, adapter=self.adapter,
                 vol_drv_iter=self._vol_drv_iter(
-                    context, instance, bdms=bdms, stg_ftsk=stg_ftsk))
+                    context, instance, bdms, stg_ftsk=stg_ftsk))
 
             # Determine if there are volumes to disconnect.  If so, remove each
             # volume (within the transaction manager)
@@ -1241,7 +1262,7 @@ class PowerVMDriver(driver.ComputeDriver):
             slot_mgr = slot.build_slot_mgr(
                 instance, self.store_api, adapter=self.adapter,
                 vol_drv_iter=self._vol_drv_iter(
-                    context, instance, bdms=bdms, stg_ftsk=stg_ftsk))
+                    context, instance, bdms, stg_ftsk=stg_ftsk))
         else:
             stg_ftsk = None
 
@@ -1639,12 +1660,9 @@ class PowerVMDriver(driver.ComputeDriver):
         mig.post_live_migration_at_destination(network_info, vol_drvs)
         del self.live_migrations[instance.uuid]
 
-    def _vol_drv_iter(self, context, instance, block_device_info=None,
-                      bdms=None, stg_ftsk=None):
+    def _vol_drv_iter(self, context, instance, bdms, stg_ftsk=None):
         """Yields a bdm and volume driver."""
         # Get a volume driver for each volume
-        if not bdms:
-            bdms = self._extract_bdm(block_device_info)
         for bdm in bdms or []:
             conn_info = bdm.get('connection_info')
             # if it doesn't have connection_info, it's not a volume
@@ -1656,13 +1674,12 @@ class PowerVMDriver(driver.ComputeDriver):
                 stg_ftsk=stg_ftsk)
             yield bdm, vol_drv
 
-    def _build_vol_drivers(self, context, instance, block_device_info=None,
-                           bdms=None, stg_ftsk=None):
+    def _build_vol_drivers(self, context, instance, block_device_info):
         """Builds the volume connector drivers for a block device info."""
         # Get a volume driver for each volume
+        bdms = self._extract_bdm(block_device_info)
         return [vol_drv for bdm, vol_drv in self._vol_drv_iter(
-            context, instance, block_device_info=block_device_info, bdms=bdms,
-            stg_ftsk=stg_ftsk)]
+            context, instance, bdms)]
 
     def unfilter_instance(self, instance, network_info):
         """Stop filtering instance."""
@@ -1720,7 +1737,7 @@ class PowerVMDriver(driver.ComputeDriver):
         lpar_uuid = vm.get_pvm_uuid(instance)
 
         # Build the connection to the VNC.
-        host = CONF.vnc.vncserver_proxyclient_address
+        host = CONF.vnc.server_proxyclient_address
         use_x509_auth = CONF.powervm.vnc_use_x509_auth
         ca_certs = CONF.powervm.vnc_ca_certs
         server_key = CONF.powervm.vnc_server_key
