@@ -37,10 +37,13 @@ FC_STRATEGY_MAPPING = {
 _STATIC_VOLUME_MAPPINGS = {
     'iscsi': 'nova_powervm.virt.powervm.volume.iscsi.'
              'IscsiVolumeAdapter',
+    'iser': 'nova_powervm.virt.powervm.volume.iscsi.'
+             'IscsiVolumeAdapter',
     'local': 'nova_powervm.virt.powervm.volume.local.'
              'LocalVolumeAdapter',
     'nfs': 'nova_powervm.virt.powervm.volume.nfs.NFSVolumeAdapter',
     'gpfs': 'nova_powervm.virt.powervm.volume.gpfs.GPFSVolumeAdapter',
+    'rbd': 'nova_powervm.virt.powervm.volume.rbd.RBDVolumeAdapter',
 }
 
 
